@@ -1,8 +1,7 @@
 """
 Pyro package. Some generic init stuff to set up logging etc.
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 import sys
@@ -59,7 +58,6 @@ config=Configuration()
 del Configuration
 
 # import the required Pyro symbols into this package
-from Pyro4.core import URI, Proxy, Daemon, callback, batch, async
+from Pyro4.core import URI, Proxy, Daemon, callback, batch, async, Future
 from Pyro4.naming import locateNS, resolve
 from Pyro4.constants import VERSION as __version__
-
