@@ -1,12 +1,11 @@
 """
 Definitions of various hard coded constants.
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 # Pyro version
-VERSION = "4.9"
+VERSION = "4.10"
 
 # standard object name for the Daemon object
 DAEMON_NAME = "Pyro.Daemon"
@@ -14,5 +13,8 @@ DAEMON_NAME = "Pyro.Daemon"
 # standard name for the Name server itself
 NAMESERVER_NAME = "Pyro.NameServer"
 
+# standard name for Flame server
+FLAME_NAME = "Pyro.Flame"
+
 # wire protocol version
 PROTOCOL_VERSION = 44
