@@ -1,14 +1,14 @@
 """
 Core logic (uri, daemon, proxy stuff).
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 from __future__ import with_statement
 import re, struct, sys, time, os
 import logging, uuid
 import hashlib, hmac
+import functools
 try:
     import copyreg
 except ImportError:
@@ -18,9 +18,9 @@ from Pyro4.socketserver.threadpoolserver import SocketServer_Threadpool
 from Pyro4.socketserver.multiplexserver import SocketServer_Select, SocketServer_Poll
 import Pyro4
 
-__all__=["URI", "Proxy", "Daemon", "callback", "batch", "async"]
+__all__=["URI", "Proxy", "Daemon", "callback", "batch", "async", "Future"]
 
-if sys.version_info>=(3, 0):
+if sys.version_info>=(3,0):
     basestring=str
 
 log=logging.getLogger("Pyro.core")
@@ -111,7 +111,7 @@ class URI(object):
 
     def __unicode__(self):
         return self.asString()
-    
+
     def __repr__(self):
         return "<%s.%s at 0x%x, %s>" % (self.__class__.__module__, self.__class__.__name__, id(self), str(self))
 
@@ -265,7 +265,7 @@ class Proxy(object):
                     data=self._pyroSerializer.deserialize(data, compressed=flags & MessageFactory.FLAGS_COMPRESSED)
                     if flags & MessageFactory.FLAGS_EXCEPTION:
                         if sys.platform=="cli":
-                            fixIronPythonExceptionForPickle(data, False) # extract
+                            fixIronPythonExceptionForPickle(data, False)
                         raise data
                     else:
                         return data
@@ -362,7 +362,6 @@ class Proxy(object):
         return self._pyroInvoke("<batch>", calls, None, flags)
 
 
-
 class _BatchedRemoteMethod(object):
     """method call abstraction that is used with batched calls"""
     def __init__(self, calls, name):
@@ -441,11 +440,11 @@ class _AsyncRemoteMethod(object):
         return _AsyncRemoteMethod(self.__proxy, "%s.%s" % (self.__name, name))
 
     def __call__(self, *args, **kwargs):
-        asyncresult=_AsyncResult()
-        thread=threadutil.Thread(target=self.__asynccall, args=(asyncresult, args, kwargs))
+        result=FutureResult()
+        thread=threadutil.Thread(target=self.__asynccall, args=(result, args, kwargs))
         thread.setDaemon(True)
         thread.start()
-        return asyncresult
+        return result
 
     def __asynccall(self, asyncresult, args, kwargs):
         try:
@@ -454,12 +453,12 @@ class _AsyncRemoteMethod(object):
             with self.__proxy.__copy__() as proxy:
                 value = proxy._pyroInvoke(self.__name, args, kwargs)
             asyncresult.value=value
-        except:
-            log.warn("exception occurred in async call, ignored")
+        except Exception:
+            # ignore any exceptions here, return them as part of the async result instead
             asyncresult.value=_ExceptionWrapper(sys.exc_info()[1])
 
 
-class _AsyncResult(object):
+class FutureResult(object):
     """
     The result object for asynchronous calls.
     """
@@ -467,6 +466,7 @@ class _AsyncResult(object):
         self.__ready=threadutil.Event()
         self.callchain=[]
         self.valueLock=threadutil.Lock()
+
     def wait(self, timeout=None):
         """
         Wait for the result to become available, with optional timeout (in seconds).
@@ -477,39 +477,92 @@ class _AsyncResult(object):
             # older pythons return None from wait()
             return self.__ready.isSet()
         return result
+
     @property
     def ready(self):
         """Boolean that contains the readiness of the async result"""
         return self.__ready.isSet()
+
     def get_value(self):
         self.__ready.wait()
         if isinstance(self.__value, _ExceptionWrapper):
             self.__value.raiseIt()
         else:
             return self.__value
+
     def set_value(self, value):
         with self.valueLock:
             self.__value=value
-            for call, kwargs in self.callchain:
-                self.__value=call(self.__value, **kwargs)
+            for call, args, kwargs in self.callchain:
+                call = functools.partial(call, self.__value)
+                self.__value = call(*args, **kwargs)
             self.callchain=[]
             self.__ready.set()
+
     value=property(get_value, set_value, None, "The result value of the call. Reading it will block if not available yet.")
-    def then(self, call, **kwargs):
+
+    def then(self, call, *args, **kwargs):
         """
-        Add a callback to the call chain, to be invoked when the results become available.
-        Optional keyword arguments in kwargs.
+        Add a callable to the call chain, to be invoked when the results become available.
+        The result of the current call will be used as the first argument for the next call.
+        Optional extra arguments can be provided in args and kwargs.
         """
         if self.__ready.isSet():
             # value is already known, we need to process it immediately (can't use the callchain anymore)
-            self.__value=call(self.__value, **kwargs)
+            call = functools.partial(call, self.__value)
+            self.__value = call(*args, **kwargs)
         else:
             # add the call to the callchain, it will be processed later when the result arrives
             with self.valueLock:
-                self.callchain.append((call,kwargs))
+                self.callchain.append((call, args, kwargs))
         return self
 
 
+class Future(object):
+    """
+    Holds a callable that will be executed asynchronously and provide its
+    result value some time in the future.
+    This is a more general implementation than the AsyncRemoteMethod, which
+    only works with Pyro proxies (and provides a bit different syntax).
+    """
+    def __init__(self, callable):
+        self.callable = callable
+        self.chain = []
+
+    def __call__(self, *args, **kwargs):
+        """
+        Start the future call with the provided arguments.
+        Control flow returns immediately, with a FutureResult object.
+        """
+        chain = self.chain
+        del self.chain  # make it impossible to add new calls to the chain once we started executing it
+        result=FutureResult()
+        thread=threadutil.Thread(target=self.__asynccall, args=(result, chain, args, kwargs))
+        thread.setDaemon(True)
+        thread.start()
+        return result
+
+    def __asynccall(self, asyncresult, chain, args, kwargs):
+        try:
+            value = self.callable(*args, **kwargs)
+            # now walk the callchain, passing on the previous value as first argument
+            for call, args, kwargs in chain:
+                call = functools.partial(call, value)
+                value = call(*args, **kwargs)
+            asyncresult.value = value
+        except Exception:
+            # ignore any exceptions here, return them as part of the async result instead
+            asyncresult.value=_ExceptionWrapper(sys.exc_info()[1])
+
+    def then(self, call, *args, **kwargs):
+        """
+        Add a callable to the call chain, to be invoked when the results become available.
+        The result of the current call will be used as the first argument for the next call.
+        Optional extra arguments can be provided in args and kwargs.
+        """
+        self.chain.append((call, args, kwargs))
+
+
 class _ExceptionWrapper(object):
     """Class that wraps a remote exception. If this is returned, Pyro will
     re-throw the exception on the receiving side. Usually this is taken care of
@@ -520,7 +573,7 @@ class _ExceptionWrapper(object):
 
     def raiseIt(self):
         if sys.platform=="cli":
-            fixIronPythonExceptionForPickle(self.exception, False) # extract attributes
+            fixIronPythonExceptionForPickle(self.exception, False)
         raise self.exception
 
 
@@ -528,6 +581,7 @@ def batch(proxy):
     """convenience method to get a batch proxy adapter"""
     return proxy._pyroBatch()
 
+
 def async(proxy):
     """convenience method to get an async proxy adapter"""
     return proxy._pyroAsync()
@@ -556,7 +610,7 @@ class MessageFactory(object):
     FLAGS_HMAC = 1<<3
     FLAGS_BATCH = 1<<4
     MAGIC = 0x34E9
-    if sys.version_info>=(3, 0):
+    if sys.version_info>=(3,0):
         empty_bytes = bytes([])
         pyro_tag = bytes("PYRO", "ASCII")
         empty_hmac = bytes(hashlib.sha1().digest_size)
@@ -933,6 +987,7 @@ class __IronPythonExceptionArgs(object):
     def __init__(self,data):
         self.data=data
 
+
 def fixIronPythonExceptionForPickle(exceptionObject, addAttributes):
     """function to hack around a bug in IronPython where it doesn't pickle
     exception attributes. We piggyback them into the exception's args."""
