@@ -1,26 +1,37 @@
 """
 Low level socket utilities.
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 import socket, os, errno, time, sys
 from Pyro4.errors import ConnectionClosedError, TimeoutError
 
+import select
+if os.name=="java":
+    selectfunction = select.cpython_compatible_select
+else:
+    selectfunction = select.select
+
 # Note: other interesting errnos are EPERM, ENOBUFS, EMFILE
 # but it seems to me that all these signify an unrecoverable situation.
 # So I didn't include them in de list of retryable errors.
-ERRNO_RETRIES=[errno.EINTR, errno.EAGAIN, errno.EWOULDBLOCK]
+ERRNO_RETRIES=[errno.EINTR, errno.EAGAIN, errno.EWOULDBLOCK, errno.EINPROGRESS]
 if hasattr(errno, "WSAEINTR"):
     ERRNO_RETRIES.append(errno.WSAEINTR)
 if hasattr(errno, "WSAEWOULDBLOCK"):
     ERRNO_RETRIES.append(errno.WSAEWOULDBLOCK)
+if hasattr(errno, "WSAEINPROGRESS"):
+    ERRNO_RETRIES.append(errno.WSAEINPROGRESS)
 
 ERRNO_BADF=[errno.EBADF]
 if hasattr(errno, "WSAEBADF"):
     ERRNO_BADF.append(errno.WSAEBADF)
 
+ERRNO_ENOTSOCK=[errno.ENOTSOCK]
+if hasattr(errno, "WSAENOTSOCK"):
+    ERRNO_ENOTSOCK.append(errno.WSAENOTSOCK)
+
 
 def getIpAddress(hostname=None):
     """returns the IP address for the current, or another, hostname"""
@@ -32,7 +43,7 @@ def getMyIpAddress(hostname=None, workaround127=False):
     it will use a little hack if the system reports our own ip address
     as being localhost (this is often the case on Linux)"""
     ip=getIpAddress(hostname)
-    if ip.startswith("127.") and workaround127:
+    if workaround127 and (ip.startswith("127.") or ip=="0.0.0.0"):
         ip=getInterfaceAddress("4.2.2.2")   # 'abuse' a level 3 DNS server
     return ip
 
@@ -164,14 +175,14 @@ def sendData(sock, data):
                 retrydelay=__nextRetrydelay(retrydelay)
 
 
-def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeout=None, noinherit=False):
+_GLOBAL_DEFAULT_TIMEOUT=object()
+
+def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeout=_GLOBAL_DEFAULT_TIMEOUT, noinherit=False):
     """
     Create a socket. Default socket options are keepalives.
     If 'bind' or 'connect' is a string, it is assumed a Unix domain socket is requested.
     Otherwise, a normal tcp/ip socket is used.
     """
-    if timeout==0:
-        timeout=None
     if bind and connect:
         raise ValueError("bind and connect cannot both be specified at the same time")
     family=socket.AF_INET
@@ -182,6 +193,10 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
         setReuseAddr(sock)
     if noinherit:
         setNoInherit(sock)
+    if timeout==0:
+        timeout = None
+    if timeout is not _GLOBAL_DEFAULT_TIMEOUT:
+        sock.settimeout(timeout)
     if bind:
         if type(bind) is tuple and bind[1]==0:
             bindOnUnusedPort(sock, bind[0])
@@ -192,17 +207,35 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
         except Exception:
             pass  # jython sometimes raises errors here
     if connect:
-        sock.connect(connect)
+        try:
+            sock.connect(connect)
+        except socket.error:
+            # This can happen when the socket is in non-blocking mode (or has a timeout configured).
+            # We check if it is a retryable errno (usually EINPROGRESS).
+            # If so, we use select() to wait until the socket is in writable state,
+            # essentially rebuilding a blocking connect() call.
+            xt, xv, xtb = sys.exc_info()
+            errno = xv.errno
+            del xt, xv, xtb
+            if errno in ERRNO_RETRIES:
+                if timeout is _GLOBAL_DEFAULT_TIMEOUT:
+                    timeout = None
+                timeout = max(0.1, timeout)   # avoid polling behavior with timeout=0
+                while True:
+                    sr, sw, se = selectfunction([], [sock], [sock], timeout)
+                    if sock in sw:
+                        break   # yay, writable now, connect() completed
+                    elif sock in se:
+                        raise socket.error("connect failed")
+            else:
+                raise
     if keepalive:
         setKeepalive(sock)
-    sock.settimeout(timeout)
     return sock
 
 
-def createBroadcastSocket(bind=None, reuseaddr=False, timeout=None):
+def createBroadcastSocket(bind=None, reuseaddr=False, timeout=_GLOBAL_DEFAULT_TIMEOUT):
     """Create a udp broadcast socket."""
-    if timeout==0:
-        timeout=None
     sock=socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
     if reuseaddr:
@@ -210,7 +243,7 @@ def createBroadcastSocket(bind=None, reuseaddr=False, timeout=None):
     if timeout is None:
         sock.settimeout(None)
     else:
-        if not bind or os.name!="java":
+        if timeout is not _GLOBAL_DEFAULT_TIMEOUT and (not bind or os.name!="java"):
             # only set timeout on the udp socket in this case, because Jython
             # has a problem with timeouts on udp sockets, see http://bugs.jython.org/issue1018
             sock.settimeout(timeout)
