@@ -1,8 +1,7 @@
 """
 Tests for the daemon.
 
-Pyro - Python Remote Objects.  Copyright by Irmen de Jong.
-irmen@razorvine.net - http://www.razorvine.net/projects/Pyro
+Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
 from __future__ import with_statement
@@ -14,6 +13,9 @@ import Pyro4.socketutil
 from Pyro4.errors import DaemonError,PyroError
 from testsupport import *
 
+Pyro4.config.reset(useenvironment=False)
+
+
 class MyObj(object):
     def __init__(self, arg):
         self.arg=arg
